/*
  Include-file    is_text_file.inc
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   TSE v4 upwards, all variants
  Version         v1   16 May 2025


  For macro programmers this include-file implements the function

    integer is_text_file(string file_name)

  It checks the named file on disk.

  It returns TRUE if the named file is a text file,
  and FALSE if it is not or if the file does not exist on disk.

  The function checks whether the named file overwhelmingly consists of
  human-readable characters. That is not a hard criterium given the lot of
  semi-readable texts out there, but the results match my expectations.


  USAGE

  - Either include is_text_file.inc in your macro with an #include statementment
    like
      #include ["is_text_file.inc"]
    or copy/paste is_text_file.inc's source code as a whole into your macro.

    Because I distribute macros, I use the copy/paste method, which avoids that
    users need to do dependency management on versions of macros and .inc files.

*/


integer is_text_file_id = 0

integer proc is_text_file(string file_name)
  integer bytes_read          = 0
  string  data [MAXSTRINGLEN] = ''
  integer handle              = 0
  integer i                   = 0
  integer org_id              = GetBufferId()
  integer result              = FALSE
  integer text_chars          = 0

  if FileExists(file_name)
    if is_text_file_id
      GotoBufferId(is_text_file_id)
      EmptyBuffer()
    else
      is_text_file_id = CreateTempBuffer()
      BinaryMode(MAXLINELEN)
    endif

    handle = fOpen(file_name)

    if handle <> -1
      repeat
        bytes_read = fRead(handle, data, MAXSTRINGLEN)
      until     bytes_read <= 0
         or not InsertText(data, _INSERT_)
         or     CurrLineLen() >= MAXLINELEN - MAXSTRINGLEN
      fClose(handle)
    endif

    for i = 1 to CurrLineLen()
      // The 0 is to accept Western UTF-16 and UTF-32 files as text.
      if (CurrChar(i) in 0, 9, 10, 12, 13, 32 .. 255)
        text_chars = text_chars + 1
      endif
    endfor

    if text_chars
      if (text_chars * 100) / CurrLineLen() >= 99   // %
        result = TRUE
      endif
    endif

    EmptyBuffer()
    GotoBufferId(org_id)
  endif

  return(result)
end is_text_file


